@@ -29,11 +29,30 @@ from .handlers import ForceReloadHandler, StaticFileHandler
 from .watcher import get_watcher_class
 from six import string_types, PY3
 
+import sys
+
+if sys.version_info >= (3, 7) or sys.version_info.major == 2:
+    import errno
+else:
+    from os import errno
+
+if sys.version_info >= (3, 8) and sys.platform == 'win32':
+    import asyncio
+    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
+
 logger = logging.getLogger('livereload')
 
 HEAD_END = b'</head>'
 
 
+def set_header(fn, name, value):
+    """Helper Function to Add HTTP headers to the server"""
+    def set_default_headers(self, *args, **kwargs):
+        fn(self, *args, **kwargs)
+        self.set_header(name, value)
+    return set_default_headers
+
+
 def shell(cmd, output=None, mode='w', cwd=None, shell=False):
     """Execute a shell command.
 
@@ -67,7 +86,7 @@ def shell(cmd, output=None, mode='w', cwd=None, shell=False):
                       shell=shell)
         except OSError as e:
             logger.error(e)
-            if e.errno == os.errno.ENOENT:  # file (command) not found
+            if e.errno == errno.ENOENT:  # file (command) not found
                 logger.error("maybe you haven't installed %s", cmd[0])
             return e
         stdout, stderr = p.communicate()
@@ -177,6 +196,25 @@ class Server(object):
             watcher_cls = get_watcher_class()
             watcher = watcher_cls()
         self.watcher = watcher
+        self.SFH = StaticFileHandler
+
+    def setHeader(self, name, value):
+        """Add or override HTTP headers at the at the beginning of the 
+           request.
+
+        Once you have intialized a server, you can add one or more 
+        headers before starting the server::
+
+            server.setHeader('Access-Control-Allow-Origin', '*')
+            server.setHeader('Access-Control-Allow-Methods', '*')
+            server.serve()
+
+        :param name: The name of the header field to be defined.
+        :param value: The value of the header field to be defined.
+        """
+        StaticFileHandler.set_default_headers = set_header(
+                StaticFileHandler.set_default_headers, name, value)
+        self.SFH = StaticFileHandler
 
     def watch(self, filepath, func=None, delay=None, ignore=None):
         """Add the given filepath for watcher list.
@@ -236,7 +274,7 @@ class Server(object):
         if liveport:
             live_script = escape.utf8(live_script % liveport)
         else:
-            live_script = escape.utf8(live_script % 'window.location.port')
+            live_script = escape.utf8(live_script % "(window.location.port || (window.location.protocol == 'https:' ? 443: 80))")
 
         web_handlers = self.get_web_handlers(live_script)
 
@@ -266,15 +304,15 @@ class Server(object):
             fallback = LiveScriptContainer(self.app, script)
             return [(r'.*', web.FallbackHandler, {'fallback': fallback})]
         return [
-            (r'/(.*)', StaticFileHandler, {
+            (r'/(.*)', self.SFH, {
                 'path': self.root or '.',
-                'default_filename': 'index.html',
+                'default_filename': self.default_filename,
             }),
         ]
 
     def serve(self, port=5500, liveport=None, host=None, root=None, debug=None,
               open_url=False, restart_delay=2, open_url_delay=None,
-              live_css=True):
+              live_css=True, default_filename='index.html'):
         """Start serve the server with the given port.
 
         :param port: serve on this port, default is 5500
@@ -287,6 +325,7 @@ class Server(object):
         :param open_url_delay: open webbrowser after the delay seconds
         :param live_css: whether to use live css or force reload on css.
                          Defaults to True
+        :param default_filename: launch this file from the selected root on startup
         """
         host = host or '127.0.0.1'
         if root is not None:
@@ -295,17 +334,18 @@ class Server(object):
         self._setup_logging()
         logger.info('Serving on http://%s:%s' % (host, port))
 
+        self.default_filename = default_filename
+
         self.application(
             port, host, liveport=liveport, debug=debug, live_css=live_css)
 
         # Async open web browser after 5 sec timeout
-        if open_url or open_url_delay:
-            if open_url:
-                logger.warn('Use `open_url_delay` instead of `open_url`')
-            sleep = open_url_delay or 5
+        if open_url:
+            logger.error('Use `open_url_delay` instead of `open_url`')
+        if open_url_delay is not None:
 
             def opener():
-                time.sleep(sleep)
+                time.sleep(open_url_delay)
                 webbrowser.open('http://%s:%s' % (host, port))
             threading.Thread(target=opener).start()
 
